diff --git a/src/backend/replication/walreceiverfuncs.c b/src/backend/replication/walreceiverfuncs.c
index 48e8498d62..301babaaa1 100644
--- a/src/backend/replication/walreceiverfuncs.c
+++ b/src/backend/replication/walreceiverfuncs.c
@@ -280,7 +280,47 @@ RequestXLogStreaming(TimeLineID tli, XLogRecPtr recptr, const char *conninfo,
 	SpinLockRelease(&walrcv->mutex);
 
 	if (launch)
+	{
 		SendPostmasterSignal(PMSIGNAL_START_WALRECEIVER);
+
+		for (;;)
+		{
+			/*
+			 *	Now we want to restart walreciver process and walrcv->walRcvState is WALRCV_STARTING.
+			 *	We should not go on startup process until walrcv->walRcvState changed to other value.
+			 *
+			 *	And we do a loop here in case of there may be any delay during restart of walreciver
+			 *	process.
+			 */
+
+			SpinLockAcquire(&walrcv->mutex);
+
+			if(WALRCV_STARTING != walrcv->walRcvState)
+			{
+				SpinLockRelease(&walrcv->mutex);
+				break;
+			}
+
+			SpinLockRelease(&walrcv->mutex);
+
+			/*
+			 * This possibly-long loop needs to handle interrupts of startup
+			 * process.
+			 */
+			HandleStartupProcInterrupts();
+
+			elog(LOG,"startup process is waiting for restart of walreciver process");
+
+			/*
+			 * The walreciver progress is restarting now, startup progress sleep 1s here to ensure
+			 * the restarting, otherwise startup progress may shutdown the restarting in function
+			 * WaitForWALToBecomeAvailable().
+			 *
+			 * Infact 1s is enought for reciver process to restart. It is related to the slave
+			 * only, noting with network. 
+			 */
+			pg_usleep(1000000L);
+		}
+	}
 	else if (latch)
 		SetLatch(latch);
 }
